@@ -4,6 +4,7 @@ from abc import ABCMeta
 from abc import abstractmethod
 from collections import namedtuple
 from pathlib import Path
+from typing import Callable
 from typing import Generator
 from typing import Iterator
 from typing import List
@@ -12,9 +13,8 @@ from typing import Type
 from typing import get_args
 from typing import get_type_hints
 
-from vortex.Tuple import TUPLE_TYPES
+from vortex.Tuple import TUPLE_TYPES_BY_NAME
 from vortex.Tuple import Tuple
-
 from .Exceptions import ItemNotFoundError
 from .Exceptions import NonUniqueNameError
 from .RelationField import RelationField
@@ -241,7 +241,11 @@ class ObjectStorageContextFileMixin(metaclass=ABCMeta):
         raise NotImplementedError()
 
     @abstractmethod
-    def downloadLargeFiles(self):
+    def downloadLargeFiles(
+        self,
+        keys: List[str],
+        archiveCompletedCallback: Callable[[str, int], None],
+    ):
         """Ensure Large Files
 
         Downloads all missing large files that have a remoteUri set
@@ -284,10 +288,15 @@ class ObjectStorageContext(ObjectStorageContextFileMixin):
                 ItemClass.storageGroup
             ] = ItemClass
 
-        cls.__registerRelationsFromClass(ItemClass, isMemberTuple=False)
+        cls.__registerRelationsFromClass(
+            ItemClass, ItemClass, isMemberTuple=False
+        )
 
         # BaseClasses are not always registered as tuples
-        if ItemClass.tupleType() and ItemClass.tupleType() in TUPLE_TYPES:
+        if (
+            ItemClass.tupleType()
+            and ItemClass.tupleType() in TUPLE_TYPES_BY_NAME
+        ):
             cls.__registerMemberTuples(ItemClass)
 
         return ItemClass
@@ -306,11 +315,13 @@ class ObjectStorageContext(ObjectStorageContextFileMixin):
                             TypingClassArg.tupleType()
                         ] = ItemClass
                         cls.__registerRelationsFromClass(
-                            TypingClassArg, isMemberTuple=True
+                            ItemClass, TypingClassArg, isMemberTuple=True
                         )
 
     @classmethod
-    def __registerRelationsFromClass(cls, TupleWithRelations, isMemberTuple):
+    def __registerRelationsFromClass(
+        cls, StorageItemClass, TupleWithRelations, isMemberTuple
+    ):
         relations = list(
             filter(
                 lambda r: isinstance(r, RelationField),
@@ -330,7 +341,7 @@ class ObjectStorageContext(ObjectStorageContextFileMixin):
                 continue
 
             # noinspection PyProtectedMember
-            relation._setOriginClass(TupleWithRelations)
+            relation._setOriginClass(StorageItemClass)
             cls._RelationGraph.add(relation)
 
     @classmethod
@@ -464,6 +475,35 @@ class ObjectStorageContext(ObjectStorageContextFileMixin):
         """
         raise NotImplementedError()
 
+    @abstractmethod
+    def getItemForExternalUuid(
+        self, group: ItemStorageGroupEnum, externalUuid: str
+    ) -> StorageTuple:
+        """Get Item For External Uuid
+
+        Get item from `group` with a particular `externalUuid`. Raises an
+        `ItemNotFoundError` if the item does not exist in the `group`.
+
+        :param group: Group to fetch item from
+        :param externalUuid: Key of the item to group
+        :return: Item with `externalUuid` from `group`
+        """
+        raise NotImplementedError()
+
+    @abstractmethod
+    def hasItemForExternalUuid(
+        self, group: ItemStorageGroupEnum, externalUuid: str
+    ) -> StorageTuple:
+        """Has Item For External Uuid
+
+        Does an item from `group` with a particular `externalUuid`.
+
+        :param group: Group to fetch item from
+        :param externalUuid: Key of the item to group
+        :return: bool: True if the item exists
+        """
+        raise NotImplementedError()
+
     @abstractmethod
     def getSingularItem(self, group: ItemStorageGroupEnum) -> StorageTuple:
         """Get Singular Item
