@@ -1,3 +1,4 @@
+import logging
 import weakref
 from abc import ABCMeta
 from collections import namedtuple
@@ -5,6 +6,10 @@ from enum import Enum
 from typing import Optional
 
 import markdown
+
+# Hide the import and loaded messages
+logging.getLogger("MARKDOWN").setLevel(logging.INFO)
+
 from vortex.Tuple import Tuple
 from vortex.Tuple import TupleField
 
@@ -59,8 +64,16 @@ class StorageTuple(Tuple, metaclass=ABCMeta):
 
     __EXPIRED__ = "expired"
 
+    #: Keys are used for internal references between objects in this project
+    # This is done to keep the project readable.
     key: NotZeroLenStr = TupleField(None, jsonExclude=False)
 
+    #: UUIDs should be referenced by systems external to this project.
+    # It should be like this because renames are hard to track in
+    # disconnected systems.
+    # This UUID will be assigned by this python package if it's missing
+    externalUuid4: Optional[str] = TupleField()
+
     def __init__(self, **kwargs):
         Tuple.__init__(self, **kwargs)
         self.__ctx = None
@@ -101,11 +114,11 @@ class StorageTuple(Tuple, metaclass=ABCMeta):
 
     @property
     def storageProject(self) -> "ContextProjectInfo":
-        return self.__ctx._projectInfo
+        return self.__ctx.info
 
     @property
     def storageKeyTuple(self) -> StorageItemKeyTuple:
-        return StorageItemKeyTuple(self.__ctx.project.key, self.key)
+        return StorageItemKeyTuple(self.storageProject.key, self.key)
 
     @classmethod
     @property
