@@ -1,13 +1,15 @@
 import json
 import logging
-import platform
+from abc import abstractmethod
 from collections import defaultdict
+from collections import namedtuple
 from datetime import datetime
 from pathlib import Path
 from typing import List
 from typing import Optional
 
 import pygit2
+import pytz
 from pathvalidate import sanitize_filepath
 from pygit2 import RemoteCallbacks
 
@@ -19,6 +21,7 @@ from ..Exceptions import NoRemoteDefined
 from ..Exceptions import ProjectValidationError
 from ..StorageTuple import ItemStorageGroupEnum
 from .._private.readme_compiler import ReadmeCompiler
+from .._private.website_compiler import WebsiteCompiler
 from ..migration import getLatestRevision
 from ..tuples.git_commit_tuple import GitCommitTuple
 from ..tuples.project_modified_tuple import ModifiedItemDetails
@@ -44,47 +47,77 @@ class PathNameInvalidException(Exception):
     pass
 
 
+GitRemote = namedtuple("GitRemote", ["name", "url"])
+
+
 class GitLibMixin:
     defaultCommitter = pygit2.Signature(
         "ServerTribe Attune", "attune@servertribe.com"
     )
 
-    def __init__(self, projectPath: Path, projectInfo: "ContextProjectInfo"):
+    def __init__(
+        self, projectPath: Path, freeGitDbResources: "ContextProjectInfo"
+    ):
         self.projectPath = projectPath
         self.__committer = self.defaultCommitter
+        self._projectInfo = freeGitDbResources
+
         self._repo = None
         self.__indexInitialised = False
+
         # TODO: What if the class was loaded before a raw commit was made?
         self.__isIndexDirty = False
 
         # Information required by the on-error context manager
         self._workingId: Optional[str] = None
 
+        self._reload()
+
+    def unload(self):
+        if self._repo:
+            self._repo.free()
+
+        self._repo = None
+        self.__indexInitialised = False
+        self.__isIndexDirty = False
+        self._workingId: Optional[str] = None
+
+    def _reload(self):
+        self.unload()
+
         try:
             self.loadGitRepo()
         except pygit2.GitError:
-            self.initGitRepo(projectInfo)
+            self.initGitRepo()
             self.loadGitRepo()
 
+    @abstractmethod
+    def load(self):
+        pass
+
     @property
     def _repoPath(self) -> Path:
         return self.projectPath / "git_storage"
 
     @property
-    def _checkedOutBranchRef(self) -> str:
+    def checkedOutBranchRef(self) -> str:
         return self._repo.head.raw_name
 
+    @property
+    def checkedOutBranchName(self) -> str:
+        return self._repo.head.name.rsplit("/", 1)[1]
+
     def loadGitRepo(self):
         self._repo = pygit2.Repository(self._repoPath.as_posix())
         if self._repo.is_bare:
             raise ValueError(f"{self._repoPath} points to a Git Bare Repo")
 
-    def initGitRepo(self, projectInfo):
+    def initGitRepo(self):
         # Create target repo
         self._repo = pygit2.init_repository(self._repoPath.as_posix())
         self.__indexInitialised = True
         logger.info(f"Create Git repository at {self._repoPath}")
-        self.__createInitialCommit(projectInfo)
+        self.__createInitialCommit(self._projectInfo)
 
     def __createInitialCommit(self, projectInfo):
         # We cannot use the public `writeMetadata` method here because the
@@ -115,6 +148,7 @@ class GitLibMixin:
                 path, self._repo.create_blob(data), pygit2.GIT_FILEMODE_BLOB
             )
         )
+        self.__index.write()
         self.__isIndexDirty = True
 
     def __writeGitIgnore(self):
@@ -126,6 +160,7 @@ class GitLibMixin:
                 path, self._repo.create_blob(data), pygit2.GIT_FILEMODE_BLOB
             )
         )
+        self.__index.write()
         self.__isIndexDirty = True
 
     def writeReadme(self, content: str):
@@ -135,6 +170,7 @@ class GitLibMixin:
         self.__index.add(
             pygit2.IndexEntry("README.md", oid, pygit2.GIT_FILEMODE_BLOB)
         )
+        self.__index.write()
         self.__isIndexDirty = True
 
     @classmethod
@@ -177,6 +213,7 @@ class GitLibMixin:
         oid = self._repo.create_blob(data)
         self.prepareWorkingBranch()
         self.__index.add(pygit2.IndexEntry(Path(path).as_posix(), oid, mode))
+        self.__index.write()
         self.__isIndexDirty = True
 
     def _moveDirectory(self, fromPath: Path, toPath: Path):
@@ -208,6 +245,7 @@ class GitLibMixin:
                 self.__index.remove(fromItemPath.as_posix())
 
         recurse(fromPath, toPath, fromTree)
+        self.__index.write()
         self.__isIndexDirty = True
 
     def _moveFile(self, fromPath: Path, toPath: Path):
@@ -225,6 +263,7 @@ class GitLibMixin:
             pygit2.IndexEntry(toPath.as_posix(), blob.oid, blob.filemode)
         )
         self.__index.remove(fromPath.as_posix())
+        self.__index.write()
         self.__isIndexDirty = True
 
     def _deleteDirectory(self, path: Path):
@@ -251,6 +290,7 @@ class GitLibMixin:
                 self.__index.remove(itemPath.as_posix())
 
         recurse(path, startTree)
+        self.__index.write()
         self.__isIndexDirty = True
 
     def _deleteFile(self, path: Path):
@@ -261,15 +301,16 @@ class GitLibMixin:
         assert isinstance(blob, pygit2.Object), "blob is not a pygit2.Object"
 
         self.__index.remove(path.as_posix())
+        self.__index.write()
         self.__isIndexDirty = True
 
     def _readFile(self, path: Path) -> bytes:
-        tree = self._getTree(path.parent)
-
-        if path.name in tree:
-            fileObject = tree / path.name
+        try:
+            fileObjectId = self.__index[str(path).strip("./").strip("/")].id
+            fileObject = self._repo[fileObjectId]
             return fileObject.data
-        else:
+
+        except KeyError:
             raise FileNotFoundError()
 
     @property
@@ -296,14 +337,23 @@ class GitLibMixin:
 
         self.__isIndexDirty = False
 
-        return self._repo.create_commit(
-            targetName,
-            author,
-            author,
-            msg,
-            tree,
-            descendents,
-        )
+        startTime = datetime.now(pytz.UTC)
+        try:
+            logger.debug("Committing to %s with message |%s|", targetName, msg)
+            return self._repo.create_commit(
+                targetName,
+                author,
+                author,
+                msg,
+                tree,
+                descendents,
+            )
+        finally:
+            logger.debug(
+                "Committing to %s completed in %s",
+                targetName,
+                (datetime.now(pytz.UTC) - startTime),
+            )
 
     @property
     def __index(self) -> pygit2.Index:
@@ -319,7 +369,7 @@ class GitLibMixin:
             )
         else:
             self._repo.index.read_tree(
-                tree=self._repo.revparse_single(self._checkedOutBranchRef).tree
+                tree=self._repo.revparse_single(self.checkedOutBranchRef).tree
             )
         self.__indexInitialised = True
 
@@ -344,11 +394,11 @@ class GitLibMixin:
 
     @property
     def __checkedOutBranch(self) -> pygit2.Branch:
-        return self._repo.references[self._checkedOutBranchRef]
+        return self._repo.references[self.checkedOutBranchRef]
 
     @property
     def __workingTree(self):
-        branch = self._checkedOutBranchRef
+        branch = self.checkedOutBranchRef
         if ATTUNE_WORKING_BRANCH in list(self._repo.branches):
             branch = makeRef(ATTUNE_WORKING_BRANCH)
 
@@ -384,8 +434,10 @@ class GitLibMixin:
     def allBranches(self) -> List[str]:
         return [str(branch) for branch in self._repo.branches]
 
-    def allRemotes(self) -> List[tuple[str, str]]:
-        return [(remote.name, remote.url) for remote in self._repo.remotes]
+    def allRemotes(self) -> List[GitRemote]:
+        return [
+            GitRemote(remote.name, remote.url) for remote in self._repo.remotes
+        ]
 
     def checkoutBranch(self, branchName: str) -> None:
         if branchName == ATTUNE_WORKING_BRANCH:
@@ -443,6 +495,7 @@ class GitLibMixin:
             )
 
         self._generateReadme()
+        WebsiteCompiler(self).generateWebsite()
         # Reload the Git _repo attribute. This will drop all references held by
         # the Repository object and allow for checkout on Windows
         self.loadGitRepo()
@@ -619,7 +672,7 @@ class GitLibMixin:
             raise Exception("There are no changes to discard")
 
         self._repo.branches.delete(ATTUNE_WORKING_BRANCH)
-        self.reloadProject()
+        self.load()
 
     def getCommits(self) -> list[GitCommitTuple]:
         for commit in self._repo.walk(self.__checkedOutBranch.target):
@@ -838,7 +891,7 @@ class GitLibMixin:
                 f"Something went wrong. Resetting __working__ to"
                 f" {self._workingId}"
             )
-            checkedBranchName = self._checkedOutBranchRef
+            checkedBranchName = self.checkedOutBranchRef
             self._repo.checkout(makeRef(ATTUNE_WORKING_BRANCH))
             self._repo.reset(self._workingId, pygit2.GIT_RESET_HARD)
             self._repo.checkout(checkedBranchName)
@@ -849,7 +902,7 @@ class GitLibMixin:
             self.__indexInitialised = False
 
             # Reload all items
-            self.reloadProject()
+            self.load()
 
     def diffCheckedOutToWorkingBranch(self) -> pygit2.Diff:
         return self._repo.diff(
@@ -857,4 +910,4 @@ class GitLibMixin:
         )
 
     def freeGitDbResources(self) -> None:
-        self._repo.free()
+        self.unload()
