@@ -1,10 +1,13 @@
 import json
 import logging
 import shutil
+import uuid
 from collections import defaultdict
 from datetime import datetime
 from pathlib import Path
+from typing import Callable
 from typing import Iterator
+from typing import List
 from typing import Optional
 
 import pygit2
@@ -12,13 +15,14 @@ import pytz
 from twisted.internet.defer import DeferredList
 from twisted.internet.defer import DeferredSemaphore
 from twisted.internet.defer import inlineCallbacks
-from vortex.Tuple import Tuple
 
 from attune_project_api.items.project_metadata_tuple import ProjectMetadataTuple
 from attune_project_api.items.step_tuples.step_sql_oracle_tuple import (
     StepSqlOracleTuple,
 )
 from attune_project_api.migration import runMigrationsForStorageContext
+from vortex.Tuple import TUPLE_TYPES_BY_NAME
+from vortex.Tuple import Tuple
 from .GitLibMixin import ATTUNE_WORKING_BRANCH
 from .GitLibMixin import GitLibMixin
 from ..Exceptions import ItemNotFoundError
@@ -33,6 +37,7 @@ from ..StorageTuple import StorageTuple
 from .._private.large_file_http_downloader import LargeFileHttpDownloader
 from .._private.utils import makeStorageKey
 from ..context_project_info import ContextProjectInfo
+from ..items import loadStorageTuples
 from ..items.file_archive_tuples.file_tuple_constants import (
     FileArchiveTupleTypeEnum,
 )
@@ -47,6 +52,7 @@ from ..tuples.project_modified_tuple import STAT_MODIFIED
 logger = logging.getLogger(__name__)
 
 DIR_CREATE_MODE = 0o700
+loadStorageTuples()
 
 
 class GitObjectStorageContextFileMixin(ObjectStorageContextFileMixin):
@@ -237,11 +243,17 @@ class GitObjectStorageContextFileMixin(ObjectStorageContextFileMixin):
         return path
 
     @inlineCallbacks
-    def downloadLargeFiles(self):
+    def downloadLargeFiles(
+        self,
+        keys: List[str],
+        archiveCompletedCallback: Callable[[str, int], None],
+    ):
         semophore = DeferredSemaphore(4)
         deferreds = []
         archivesToBeDownloaded = []
-        for item in self.getItems(ItemStorageGroupEnum.FileArchive):
+        for key in keys:
+            item = self.getItem(ItemStorageGroupEnum.FileArchive, key)
+
             if item.tupleType() != FileArchiveTupleTypeEnum.LARGE.value:
                 continue
 
@@ -256,7 +268,7 @@ class GitObjectStorageContextFileMixin(ObjectStorageContextFileMixin):
 
             archivesToBeDownloaded.append(item)
             downloader = LargeFileHttpDownloader(
-                item.remoteUri, item.archivePath
+                key, item.remoteUri, item.archivePath, archiveCompletedCallback
             )
             deferreds.append(semophore.run(downloader.download))
 
@@ -274,6 +286,8 @@ class GitObjectStorageContextFileMixin(ObjectStorageContextFileMixin):
 class GitObjectStorageContext(
     GitLibMixin, ObjectStorageContext, GitObjectStorageContextFileMixin
 ):
+    _DEBUG_ENABLED = False
+
     def __init__(self, projectPath: Path, info: ContextProjectInfo):
         assert isinstance(projectPath, Path), "projectPath is not of type Path"
         self._projectInfo = info
@@ -284,15 +298,28 @@ class GitObjectStorageContext(
                 ensuring that "one and only one" reference exists for an item
                 For example, s1 = getItem("Step1"), s2 = getItem("Step1")
                 id(s1) == id(s2) """
-        self.__instanceByKeyByGroup = defaultdict(dict)
+        self.__instanceByKeyByGroup: dict[
+            ItemStorageGroupEnum, dict[str, StorageTuple]
+        ] = defaultdict(dict)
 
-        self.__instanceByKeyByGroup[ItemStorageGroupEnum.Step] = {}
-        self.__instanceByKeyByGroup[ItemStorageGroupEnum.Parameter] = {}
-        self.__instanceByKeyByGroup[ItemStorageGroupEnum.FileArchive] = {}
+        self.__instanceByUuidByGroup: dict[
+            ItemStorageGroupEnum, dict[str, StorageTuple]
+        ] = defaultdict(dict)
 
-        self.__singularInstanceGroup = defaultdict(lambda: None)
+        self.__singularInstanceGroup: dict[
+            ItemStorageGroupEnum, Optional[StorageTuple]
+        ] = defaultdict(lambda: None)
+
+        self._migrateProjectMetadata(info)
+
+    @property
+    def info(self) -> ContextProjectInfo:
+        return self._projectInfo
+
+    def _migrateProjectMetadata(self, info):
         self.__singularInstanceGroup[ItemStorageGroupEnum.Project] = None
-        self.__loadSingularItem(ItemStorageGroupEnum.Project)
+
+        self.__loadSingularItem(ItemStorageGroupEnum.Project, logError=False)
 
         revision = self.metadata.revision
         if not checkIfLibrarySupportsRevision(revision):
@@ -302,14 +329,18 @@ class GitObjectStorageContext(
             )
 
         # Reload project metadata incase the migrations modified it
-        self.__loadSingularItem(ItemStorageGroupEnum.Project)
+        self.__loadSingularItem(ItemStorageGroupEnum.Project, logError=False)
 
+        # Ensure the metadata is populated
+        if not self.getSingularItem(ItemStorageGroupEnum.Project):
+            self.addItem(self.__metadataLoadFallback())
+
+    # noinspection PyStatementEffect,PyTypeChecker
     def load(self):
-        # Run the migrations to bring the project to the latest revision
-        # Migrations need to be run before the items are loaded and as a
-        # result cannot use the public methods (getItem, addItem) in the
-        # `upgrade` and `downgrade` functions
-        runMigrationsForStorageContext(self)
+        self._reload()
+        self.__instanceByKeyByGroup = defaultdict(dict)
+        self.__instanceByUuidByGroup = defaultdict(dict)
+        self.__singularInstanceGroup = defaultdict(lambda: None)
 
         # Cache all items in memory upon load.
         # This will speed up the access of the data
@@ -317,23 +348,25 @@ class GitObjectStorageContext(
         self.__loadItems(ItemStorageGroupEnum.FileArchive)
         self.__loadItems(ItemStorageGroupEnum.Step)
 
-    def reloadProject(self) -> None:
-        self.__instanceByKeyByGroup[ItemStorageGroupEnum.Step] = {}
-        self.__instanceByKeyByGroup[ItemStorageGroupEnum.Parameter] = {}
-        self.__instanceByKeyByGroup[ItemStorageGroupEnum.FileArchive] = {}
-        self.__singularInstanceGroup[ItemStorageGroupEnum.Project] = None
+        if not self.__singularInstanceGroup[ItemStorageGroupEnum.Project]:
+            self.addItem(self.__metadataLoadFallback())
+            self.commit("Upgraded an wrote project metadata")
 
-        self.__loadItems(ItemStorageGroupEnum.Parameter)
-        self.__loadItems(ItemStorageGroupEnum.FileArchive)
-        self.__loadItems(ItemStorageGroupEnum.Step)
+        # Run the migrations to bring the project to the latest revision
+        # Migrations need to be run before the items are loaded and as a
+        # result cannot use the public methods (getItem, addItem) in the
+        # `upgrade` and `downgrade` functions
+        runMigrationsForStorageContext(self)
 
         self.__loadSingularItem(ItemStorageGroupEnum.Project)
 
+        if self.isDirty:
+            self.commit("Automatically assigned missing UUID4s")
+
     @property
     def metadata(self) -> ProjectMetadataTuple:
         metadata = self.__singularInstanceGroup[ItemStorageGroupEnum.Project]
-        if metadata is None:
-            metadata = self.__metadataLoadFallback()
+        assert metadata, "Metadata is None"
         return metadata
 
     def __metadataLoadFallback(self):
@@ -370,14 +403,45 @@ class GitObjectStorageContext(
         assert group in self.__instanceByKeyByGroup, (
             "Group %s is not a valid" % group
         )
-        itemsByKey = self.__instanceByKeyByGroup[group]
 
-        if key not in itemsByKey:
+        try:
+            return self.__instanceByKeyByGroup[group][key]
+
+        except KeyError:
             raise ItemNotFoundError(
                 "Item not found, Group [%s], Key [%s]" % (group, key)
             )
 
-        return itemsByKey[key]
+    def getItemForExternalUuid(
+        self, group: ItemStorageGroupEnum, externalUuid: str
+    ) -> StorageTuple:
+        assert group in self.__instanceByUuidByGroup, (
+            "Group %s is not a valid" % group
+        )
+
+        try:
+            return self.__instanceByUuidByGroup[group][externalUuid]
+
+        except KeyError:
+            raise ItemNotFoundError(
+                "Item not found, Group [%s], UUID [%s]" % (group, externalUuid)
+            )
+
+    def hasItemForKey(self, group: ItemStorageGroupEnum, itemKey: str) -> bool:
+        assert group in self.__instanceByUuidByGroup, (
+            "Group %s is not a valid" % group
+        )
+
+        return itemKey in self.__instanceByKeyByGroup[group]
+
+    def hasItemForExternalUuid(
+        self, group: ItemStorageGroupEnum, externalUuid: str
+    ) -> bool:
+        assert group in self.__instanceByUuidByGroup, (
+            "Group %s is not a valid" % group
+        )
+
+        return externalUuid in self.__instanceByUuidByGroup[group]
 
     def getSingularItem(
         self, group: ItemStorageGroupEnum
@@ -393,7 +457,7 @@ class GitObjectStorageContext(
         group: ItemStorageGroupEnum,
     ) -> Iterator[StorageTuple]:
         assert group in self.__instanceByKeyByGroup, (
-            "Group %s is not " "valid" % group
+            "Group %s is not valid" % group
         )
         return iter(self.__instanceByKeyByGroup[group].values())
 
@@ -437,8 +501,8 @@ class GitObjectStorageContext(
 
     def reloadItem(self, item: StorageTuple) -> StorageTuple:
         freshItem = self.__readItem(item.__class__, item.key)
-        self.__storeReferenceToTuple(freshItem)
         freshItem._bind(self)
+        self.__storeReferenceToTuple(freshItem)
         item._expire()
         return freshItem
 
@@ -515,18 +579,78 @@ class GitObjectStorageContext(
         assert (
             item.storageContext is self
         ), "Item must be bound before it is cached"
-        if item.__allowsMultiple__:
-            self.__instanceByKeyByGroup[item.storageGroup][item.key] = item
-        else:
+
+        if not item.__allowsMultiple__:
             self.__singularInstanceGroup[item.storageGroup] = item
+            oldItem = self.__singularInstanceGroup[item.storageGroup]
+
+        else:
+            oldItem = self.__instanceByKeyByGroup[item.storageGroup].pop(
+                item.key, None
+            )
+            if oldItem and oldItem.externalUuid4:
+                self.__instanceByUuidByGroup[oldItem.storageGroup].pop(
+                    oldItem.externalUuid4, None
+                )
+
+        # If we have an old UUID, **ALWAYS** keep that.
+        if (
+            oldItem
+            and oldItem.externalUuid4
+            and oldItem.externalUuid4 != item.externalUuid4
+        ):
+            item.externalUuid4 = oldItem.externalUuid4
+            logger.debug(
+                "Assigning UUID4 %s from old %s with key %s",
+                item.externalUuid4,
+                item.storageGroup.value,
+                item.key,
+            )
+
+        elif not item.externalUuid4:
+            item.externalUuid4 = str(uuid.uuid4())
+            logger.debug(
+                "Assigning UUID4 %s to %s with key %s",
+                item.externalUuid4,
+                item.storageGroup.value,
+                item.key,
+            )
+
+            self.__writeItem(item, metadataOnly=True)
+
+        if not item.__allowsMultiple__:
+            self.__singularInstanceGroup[item.storageGroup] = item
+
+        else:
+            if (
+                item.externalUuid4
+                in self.__instanceByUuidByGroup[item.storageGroup]
+            ):
+                raise Exception(
+                    f"Duplicate UUID4 {item.externalUuid4} "
+                    f"detected in {item.storageGroup.value}"
+                )
+
+            self.__instanceByKeyByGroup[item.storageGroup][item.key] = item
+            self.__instanceByUuidByGroup[item.storageGroup][
+                item.externalUuid4
+            ] = item
 
     def __removeReferenceToTuple(self, item):
         if item.__allowsMultiple__:
             self.__instanceByKeyByGroup[item.storageGroup].pop(item.key, None)
+            self.__instanceByUuidByGroup[item.storageGroup].pop(
+                item.externalUuid4, None
+            )
         else:
             self.__singularInstanceGroup.pop(item.storageGroup)
 
     def __loadItems(self, group: ItemStorageGroupEnum) -> None:
+        # noinspection PyStatementEffect
+        self.__instanceByKeyByGroup[group]
+        # noinspection PyStatementEffect
+        self.__instanceByUuidByGroup[group]
+
         startTime = datetime.now(pytz.utc)
         try:
             tree = self._getTree(Path(group.value))
@@ -545,6 +669,8 @@ class GitObjectStorageContext(
             try:
                 item = self.__readItem(BaseClass, object_tree.name)
             except (TypeError, ItemNotFoundError) as e:
+                if self._DEBUG_ENABLED:
+                    logger.exception(e)
                 errors.append(
                     "Group %s, Key %s, Error: %s"
                     % (group.value, object_tree.name, str(e))
@@ -560,12 +686,22 @@ class GitObjectStorageContext(
             raise ProjectValidationError(errors)
 
         logger.debug(
-            "Completed loading group %s in %s",
+            "Completed loading %s items in group %s in %s",
+            (
+                len(self.__instanceByKeyByGroup[group])
+                if BaseClass.__allowsMultiple__
+                else (1 if self.__singularInstanceGroup[group] else 0)
+            ),
             group,
             datetime.now(pytz.utc) - startTime,
         )
 
-    def __loadSingularItem(self, group: ItemStorageGroupEnum) -> None:
+    def __loadSingularItem(
+        self, group: ItemStorageGroupEnum, logError=True
+    ) -> None:
+        # noinspection PyStatementEffect
+        self.__singularInstanceGroup[group]
+
         try:
             tree = self._getTree(Path(group.value))
         except FileNotFoundError:
@@ -579,27 +715,21 @@ class GitObjectStorageContext(
         try:
             item = self.__readItem(BaseClass)
         except Exception as e:
-            logger.debug(f"Error in loading group {group.value}: {str(e)}")
+            if logError:
+                logger.exception(e)
+                logger.debug(f"Error in loading group {group.value}: {str(e)}")
             return None
 
         item._bind(self)
         self.__storeReferenceToTuple(item)
 
-    def __writeItem(self, item: type[StorageTuple]) -> None:
+    def __writeItem(self, item: StorageTuple, metadataOnly=False) -> None:
         fields = item.tupleToRestfulJsonDict()
         comment = fields.pop("comment", None)
         # Scripts and SQL is stored under a `script.txt` file
         script = fields.pop("script", None) or fields.pop("sql", None)
 
-        # FIXME If the name isn't suitable for storage
-        #  then an exception needs raising instead.
-        sanitizedItemKey = makeStorageKey(item.key)
-        group = item.storageGroup.value
-
-        if not item.__allowsMultiple__:
-            itemPath = Path(f"{group}")
-        else:
-            itemPath = Path(f"{group}/{sanitizedItemKey}")
+        itemPath = self.__makeItemPath(item)
 
         self._writeFile(
             (itemPath / "metadata.json").as_posix(),
@@ -611,6 +741,9 @@ class GitObjectStorageContext(
             ).encode(),
         )
 
+        if metadataOnly:
+            return
+
         if comment:
             self._writeFile(
                 (itemPath / "README.md").as_posix(), comment.encode()
@@ -630,6 +763,21 @@ class GitObjectStorageContext(
                 (itemPath / "script.txt").as_posix(), script.encode()
             )
 
+    def __makeItemPath(self, item):
+        if item.key != makeStorageKey(item.key):
+            logger.warning(
+                "item key is invalid, it must be the result of "
+                "makeStorageKey(item.key), Expected %s, got %s",
+                makeStorageKey(item.key),
+                item.key,
+            )
+
+        if not item.__allowsMultiple__:
+            itemPath = Path(f"{item.storageGroup.value}")
+        else:
+            itemPath = Path(f"{item.storageGroup.value}/{item.key}")
+        return itemPath
+
     def __moveItemDirectory(
         self, group: ItemStorageGroupEnum, oldKey: str, newKey: str
     ):
@@ -681,26 +829,35 @@ class GitObjectStorageContext(
             data = self._readFile(path / "metadata.json")
             metadataStr = data.decode()
             metadata = json.loads(metadataStr)
-        except FileNotFoundError:
+        except FileNotFoundError as e:
+            if self._DEBUG_ENABLED:
+                logger.exception(e)
             raise ItemNotFoundError(f"{key} does not exist in {groupStr}")
 
         try:
             commentData = self._readFile(path / "README.md")
             metadata["comment"] = commentData.decode()
         except FileNotFoundError:
-            metadata["comment"] = None  # Most tuples default this anyways
+            metadata["comment"] = None  # Most tuples default this anyway
 
-        try:
-            scriptData = self._readFile(path / "script.txt")
-            # Not every item has target 'script' or `sql` associated with it
-            if scriptData is not None:
-                metadata[
-                    "sql"
-                    if metadata["type"] == StepSqlOracleTuple.tupleType()
-                    else "script"
-                ] = scriptData.decode()
-        except FileNotFoundError:
-            pass
+        # Check for a script, only if out step type has one
+        if "type" in metadata and metadata["type"] in TUPLE_TYPES_BY_NAME:
+            ConcreteItemClass = TUPLE_TYPES_BY_NAME[metadata["type"]]
+            if hasattr(ConcreteItemClass, "script") or hasattr(
+                ConcreteItemClass, "sql"
+            ):
+                try:
+                    scriptData = self._readFile(path / "script.txt")
+                    # Not every item has target 'script' or `sql` associated with it
+                    if scriptData is not None:
+                        metadata[
+                            "sql"
+                            if metadata["type"]
+                            == StepSqlOracleTuple.tupleType()
+                            else "script"
+                        ] = scriptData.decode()
+                except FileNotFoundError:
+                    pass
 
         # noinspection PyTypeChecker
         return Tuple.restfulJsonDictToTupleWithValidation(metadata, ItemClass)
@@ -721,7 +878,7 @@ class GitObjectStorageContext(
             raise Exception("There are no changes to discard")
 
         self._repo.branches.delete(ATTUNE_WORKING_BRANCH)
-        self.reloadProject()
+        self.load()
 
     def renameProject(self, newName: str):
         """
