@@ -1,5 +1,6 @@
 import logging
 from pathlib import Path
+from typing import Callable
 from typing import Union
 
 import certifi
@@ -19,11 +20,24 @@ from twisted.web.http_headers import Headers
 
 logger = logging.getLogger(__name__)
 
+# These are mirrored from Attune backend and front-end
+IN_PROGRESS = 1
+FAILED = 2
+COMPLETED = 3
+
 
 class LargeFileHttpDownloader:
-    def __init__(self, url: Union[str, bytes], path: Path):
+    def __init__(
+        self,
+        archiveKey: str,
+        url: Union[str, bytes],
+        path: Path,
+        archiveCompletedCallback: Callable[[str, int], None],
+    ):
         self._path = path
         self._url = url.encode() if isinstance(url, str) else url
+        self._archiveKey = archiveKey
+        self._archiveCompletedCallback = archiveCompletedCallback
 
     @inlineCallbacks
     def download(self) -> Deferred:
@@ -57,8 +71,13 @@ class LargeFileHttpDownloader:
                     continue
 
                 if response.code == 200:
-                    downloader = _FileDownloader(self._path)
+                    downloader = _FileDownloader(
+                        self._path,
+                        self._archiveKey,
+                        self._archiveCompletedCallback,
+                    )
                 else:
+                    self._archiveCompletedCallback(self._archiveKey, FAILED)
                     downloader = _BodyError(
                         response.code, response.request.absoluteURI
                     )
@@ -70,11 +89,19 @@ class LargeFileHttpDownloader:
 
 
 class _FileDownloader(Protocol):
-    def __init__(self, path: Path):
+    def __init__(
+        self,
+        path: Path,
+        archiveKey: str,
+        archiveCompletedCallback: Callable[[str, int], None],
+    ):
         self._finishedDeferred = Deferred()
         self._file = open(path, "wb")
         self._writeSize = 0
 
+        self._archiveKey = archiveKey
+        self._archiveCompletedCallback = archiveCompletedCallback
+
     @property
     def deferred(self) -> Deferred:
         return self._finishedDeferred
@@ -87,9 +114,11 @@ class _FileDownloader(Protocol):
         self._file.close()
 
         if isinstance(reason.value, ResponseDone):
+            self._archiveCompletedCallback(self._archiveKey, COMPLETED)
             self._finishedDeferred.callback(self._file)
             return
 
+        self._archiveCompletedCallback(self._archiveKey, FAILED)
         self._finishedDeferred.errback(reason)
 
 
