@@ -22,7 +22,11 @@ PATH = Path("metadata.json")
 
 
 def upgrade(storageContext: GitObjectStorageContext):
-    readme = json.loads(storageContext._readFile(PATH).decode())
+    try:
+        readme = json.loads(storageContext._readFile(PATH).decode())
+    except FileNotFoundError:
+        readme = {}
+
     projectInfo = storageContext._projectInfo
 
     readme["name"] = projectInfo.name
