@@ -17,16 +17,9 @@ from zipfile import ZIP_DEFLATED
 from zipfile import ZIP_STORED
 
 import pytz
-from pytmpdir.spooled_named_temporary_file import SpooledNamedTemporaryFile
-from vortex.DeferUtil import deferToThreadWrapWithLogger
-from vortex.DeferUtil import noMainThread
-from vortex.Tuple import addTupleType
-
+from attune_project_api import ObjectStorageContext
 from attune_project_api.ObjectStorageContext import VersionedFileContent
 from attune_project_api.ObjectStorageContext import VersionedFileInfo
-from attune_project_api._contexts.GitObjectStorageContext import (
-    GitObjectStorageContext,
-)
 from attune_project_api.items.file_archive_tuples.file_archive_tuple import (
     FileArchiveTuple,
 )
@@ -39,11 +32,15 @@ from attune_project_api.items.file_archive_tuples.file_tuple_constants import (
 from attune_project_api.items.file_archive_tuples.file_tuple_constants import (
     FileArchiveTupleTypeEnum,
 )
+from pytmpdir.spooled_named_temporary_file import SpooledNamedTemporaryFile
+from vortex.DeferUtil import deferToThreadWrapWithLogger
+from vortex.DeferUtil import noMainThread
+from vortex.Tuple import addTupleType
 
 logger = logging.getLogger(__name__)
 
 
-@GitObjectStorageContext.registerItemClass
+@ObjectStorageContext.registerItemClass
 @addTupleType
 class FileArchiveVersionedTuple(FileArchiveTuple):
     """Archive Versioned Tuple
