@@ -1,11 +1,10 @@
 from typing import Optional
 
-from vortex.Tuple import TupleField
-from vortex.Tuple import addTupleType
-
 from attune_project_api import ObjectStorageContext
 from attune_project_api import StorageTuple
 from attune_project_api.StorageTuple import ItemStorageGroupEnum
+from vortex.Tuple import TupleField
+from vortex.Tuple import addTupleType
 
 
 @ObjectStorageContext.registerItemClass
@@ -19,3 +18,7 @@ class ProjectMetadataTuple(StorageTuple):
 
     revision: Optional[str] = TupleField(defaultValue="16e34a42a9a1")
     comment: Optional[str] = TupleField()
+
+    @property
+    def uuid(self) -> str:
+        return self.externalUuid4
