@@ -11,12 +11,13 @@
 """
 import logging
 
-from vortex.Tuple import TupleField
-from vortex.Tuple import addTupleType
-
+from attune_project_api import ObjectStorageContext
 from attune_project_api.items.file_archive_tuples.file_archive_tuple import (
     FileArchiveTuple,
 )
+from vortex.Tuple import TupleField
+from vortex.Tuple import addTupleType
+
 from . import addStepDeclarative
 from . import extractTextPlaceholders
 from .step_tuple import StepTuple
@@ -25,7 +26,6 @@ from .. import NotZeroLenStr
 from ..parameter_tuple import ParameterTuple
 from ...RelationField import RelationField
 from ...StorageTuple import StorageMemberTuple
-from ..._contexts.GitObjectStorageContext import GitObjectStorageContext
 
 logger = logging.getLogger(__name__)
 
@@ -48,7 +48,7 @@ class StepPushDesignFileCompiledParamTuple(StorageMemberTuple):
     )
 
 
-@GitObjectStorageContext.registerItemClass
+@ObjectStorageContext.registerItemClass
 @addStepDeclarative("Push Compiled Files")
 @addTupleType
 class StepPushDesignFileCompiledTuple(StepTuple):
@@ -78,6 +78,14 @@ class StepPushDesignFileCompiledTuple(StepTuple):
     makoFileNames: list[str] = TupleField([])
 
     makoParameters: list[StepPushDesignFileCompiledParamTuple] = TupleField([])
+    storageParameters: list[StepTuple] = RelationField(
+        ForeignClass=ParameterTuple,
+        referenceKeyFieldName="makoParameters",
+        isList=True,
+        cascadeOnDelete=True,
+        cascadeOnUpdate=True,
+        memberReferenceKeyFieldName="parameterKey",
+    )
 
     def parameters(self) -> list["ParameterTuple"]:
         return [self.server, self.osCred] + [
