@@ -1,9 +1,6 @@
-from vortex.Tuple import TupleField
-from vortex.Tuple import addTupleType
-
+from attune_project_api import ObjectStorageContext
 from attune_project_api import ParameterTuple
 from attune_project_api.RelationField import RelationField
-from attune_project_api._contexts import GitObjectStorageContext
 from attune_project_api.items import NotZeroLenStr
 from attune_project_api.items.file_archive_tuples.file_archive_tuple import (
     FileArchiveTuple,
@@ -12,9 +9,11 @@ from attune_project_api.items.step_tuples import addStepDeclarative
 from attune_project_api.items.step_tuples import extractTextPlaceholders
 from attune_project_api.items.step_tuples.step_tuple import StepTuple
 from attune_project_api.items.step_tuples.step_tuple import StepTupleTypeEnum
+from vortex.Tuple import TupleField
+from vortex.Tuple import addTupleType
 
 
-@GitObjectStorageContext.registerItemClass
+@ObjectStorageContext.registerItemClass
 @addStepDeclarative("Push Files")
 @addTupleType
 class StepPushDesignFileTuple(StepTuple):
